% This program generates draws for variance decomposition 
%%% By Christiane Baumeister and James D. Hamilton (Aug 2015)

% To be run after running irf.m, which should have set variables including:
%  psi_invA = [n x n x (hmax+nlags) x nuse] array which includes structural
%     impulse-response matrices
%     dimension 1: variable affected
%     dimension 2: shock whose effect is sought
%     dimension 3: hmax + s component is effect of shock after s periods (s = 0,1,..,hmax-1)
%     dimension 4: indexes the Monte Carlo draw of the parameters
% note psi_invA is the cumulation of IRFs if kcum = 1
%  invdDraw = (n x nuse) array of inverse of structural variances
%     invdDraw(i,m) = inverse of variance of ith structural equation associated with mth parameter draw

% output:
% Q = [n x n x (hmax+nlags) x nuse] array of total MSE coming from each shock
%     dimension 1: variable affected
%     dimension 2: shock whose effect is sought
%     dimension 3: hmax + s component is MSE of s-period-ahead forecast (s = 0,1,..,hmax-1)
%     dimension 4: indexes the Monte Carlo draw of the parameters



Q = zeros(n,n,hmax+nlags,nuse);

% ===========================================================
% 'Generating draws for variance decomposition'
isim = 0;
while isim < nuse
    isim = isim + 1;
    if (isim/10000) == floor(isim/10000)
        isim
    end
    hsim = 0;
    while hsim < hmax
        H = squeeze(psi_invA0(:,:,nlags+hsim,isim));
        % psi_invA0 has cumulated IRF  when kcum = 1, regular when kcum = 0
        j = 0;
        while j < n
            j = j+1;
            Q(:,j,nlags+hsim,isim) = Q(:,j,nlags+hsim-1,isim) + (1/invdDraw(j,isim))*H(:,j).*H(:,j);
        end
    hsim = hsim +1;
    end
end

% ============================================================
% Report results
shocknames2 = char('supply','demand','monetary policy');
display('Rows correspond to variance decomposition of different variables')
display('columns to effects of different shocks')
index0 = round(0.5*nuse);
index1 = round(0.025*nuse);
index2 = round((1 - 0.025)*nuse);

H0 = zeros(n,n,hmax);
H1 = zeros(n,n,hmax);
H2 = zeros(n,n,hmax);
    for i = 1:n
        for j = 1:n
           K = squeeze(Q(i,j,nlags:nlags+hmax-1,:));
           K = sort(K,2);
           H0(i,j,:) = K(:,index0);
           H1(i,j,:) = K(:,index1);
           H2(i,j,:) = K(:,index2);
        end
    end

% calculate and report final results   
for s = 0:hmax-1
    % reformulate as fractions
    %for i=1:n
      % H0(i,:,s+1) = H0(i,:,s+1)./squeeze(sum(H0(i,:,s+1),2))';
      % H1(i,:,s+1) = H1(i,:,s+1)./squeeze(sum(H1(i,:,s+1),2))';
       %H2(i,:,s+1) = H2(i,:,s+1)./squeeze(sum(H2(i,:,s+1),2))';
    %end
    display(['Horizon ' num2str(s+1)])
    display('median')
    display(H0(:,:,s+1))
    display('lower 2.5%')
    display(H1(:,:,s+1))
    display('upper 2.5%')
    display(H2(:,:,s+1))
end

